#! /usr/bin/env python3
#
# Deploy Radiostations for Zynthian's iNetRadio PD-patch
#
# Arguments:
#   accepts one argument: radio-stations file (default = radiostations.txt)
#
# reads radio-station names and stream-urls from radiostations.txt
# unused radio-station slots will be set to "Off"
# reads main.pd.template, inserts stream-urls and write out to main.pd
# reads zynconfig.yml.template, inserts station names and write out to zynconfig.yml
#
# ATTENTION:  The files main.pd and zynconfig.yml will get overwritten!
#             This script does no error-checking! 
#
#
# script revision: 2024-02-04


import sys
import argparse
import string
import re

# default name of the file containing radio-station names and stream-urls
stations_filename = 'radiostations.txt'

# how many slots for radio stations are in main.pd.template?
max_stations = 32

# Process command line arguments
parser = argparse.ArgumentParser(description='Deploy Radiostations for Zynthian\'s iNetRadio PD-patch.')
parser.add_argument('stationsfile', nargs='?', default=stations_filename, 
                    help='text-file containing the station names and stream URL\'s to be deployed. (default=' + stations_filename + ')')
parser.add_argument('-n', '--stations', metavar='N', type=int, default=max_stations,
                    help='number of station slots to deploy')
parser.add_argument('-d', '--defaultstation', metavar='N', type=int, default=1,
                    help='station-number to load on startup')
parser.add_argument('-a', '--altprefix', action='store_true',
                    help='alternative station prefix (default="[NUM]", alternative="NUM:")')
parser.add_argument('-s', '--speed', action='store_true',
                    help='enable speed control')
parser.add_argument('-p', '--preview', action='store_true',
                    help='stations preview only. Does not write out any files')
parser.add_argument('-v', '--verbose', action='store_true',
                    help='print internals')

args = parser.parse_args()

# Validity checks of some arguments
if args.stations > max_stations:
    print('number of stations given in argument is higher than number of station slots!')
    sys.exit(1)
if args.defaultstation > args.stations:
    print('defaultstation number is higher than number of stations!')
    sys.exit(1)
defaultstation = args.defaultstation -1 if args.defaultstation > 0 else 0

# Open the radiostations file
try:
    stations_file = open(args.stationsfile, 'r')
except Exception as exc:
    print("Radio-stations file: " + str(exc))
    sys.exit(1)

# Read in the main.pd.template file
try:
    with open('main.pd.template', 'r') as mainpd_file:
        mainpd_data = mainpd_file.read()
except Exception as exc:
    print(str(exc))
    sys.exit(1)

# Read in the zynconfig.yml.template file
try:
    with open('zynconfig.yml.template', 'r') as config_file:
        config_data = config_file.read()
except Exception as exc:
    print(str(exc))
    sys.exit(1)

# Replace placeholder DIVISOR in main.pd
divisor = 127 / (args.stations - 1)
divisor = str(round(divisor, 2))
mainpd_data = mainpd_data.replace('DIVISOR', divisor)

if args.verbose:
    print('Number of radio stations: ' + str(args.stations))
    print('Default station number on startup: ' + str(defaultstation+1))
    print('DIVISOR for Station-CC: ' + divisor)

# Disable speed control in zynconfig.yml
if not args.speed:
    config_data = re.sub(r'(?s)# SPEED_START.+# SPEED_END\n', '', config_data)
else:
    config_data = config_data.replace('# SPEED_START\n', '')
    config_data = config_data.replace('# SPEED_END\n', '')


# Deploy the radio stations to the PD-patch and the zynconfig file
for i in range(args.stations):
    station_name = stations_file.readline()
    if args.altprefix:
        station_prefix = str(i+1) + ': '
    else:
        station_prefix = '[' + str(i+1) + '] '
    # skip blank lines and remark lines (but prevent an endless loop at EOF)  
    while (station_name != "" and station_name.strip() == "") or station_name[:1] == "#":
        station_name = stations_file.readline()
    station_url = stations_file.readline()
    # skip remark lines
    while station_url[:1] == "#":
        station_url = stations_file.readline()
    station_name = station_name.strip()
    station_url = station_url.strip()
    if not station_url or station_url[:1] == "#":
        station_name = "Off"
        station_url = "Off"

    # Replace the target strings
    config_data = config_data.replace('NAME#'+str(i).zfill(2), station_prefix + station_name)
    mainpd_data = mainpd_data.replace('STREAM#'+str(i).zfill(2), station_url)
    if i == defaultstation: 
        config_data = config_data.replace('DEFAULTSTATION', station_prefix + station_name)
        mainpd_data = mainpd_data.replace('DEFAULTSTATION', str(i))

    # Print a list of the configured radio-stations
    print(station_prefix + station_name + ' --> ' + station_url)

# Delete excessive stations in the zynconfig file
for i in range(args.stations, max_stations):
    label = "'NAME#"+str(i).zfill(2)+"'"
    config_data = config_data.replace(', '+label, '')

    # Print a list of the removed excessive station slots
    if args.verbose:
        print(str(i+1) + ': slot removed '+label)

# Write the files out
if not args.preview:
    try:
        with open('main.pd', 'w') as file:
          file.write(mainpd_data)
        with open('zynconfig.yml', 'w') as file:
          file.write(config_data)
    except Exception as exc:
        print(str(exc))


